# Copyright (c) 2011 gocept gmbh & co. kg
# See also LICENSE.txt

import datetime


class LogFile(object):

    def __init__(self, log_file):
        if isinstance(log_file, str):
            log_file = open(log_file)
        self.log_file = log_file

    def parse(self):
        result = []
        for line in self.log_file.readlines():
            parts = line.strip().split(' ')
            result.append(Record(
                # XXX handle timezone info in parts[1]
                date=self._parse_date(parts[0]),
                url=parts[3],
                time=float(parts[-1])))
        return result

    def _parse_date(self, date):
        return datetime.datetime.strptime(date, '%d/%b/%Y:%H:%M:%S')

    def records_since(self, last_seen_date):
        return [x for x in self.parse() if x.date > last_seen_date]


class Record(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)


class Statistics(object):

    def __init__(self, records):
        # assumption: records are sorted chronologically
        self.records = records
        self.times = [x.time for x in self.records]

    @property
    def min(self):
        return min(self.times)

    @property
    def max(self):
        return max(self.times)

    @property
    def mean(self):
        return sum(self.times) / self.count

    @property
    def stddev(self):
        squares = sum([(i - self.mean) ** 2 for i in self.times])
        return (squares / (self.count - 1)) ** 0.5

    @property
    def count(self):
        return len(self.records)

    @property
    def last_date(self):
        return self.records[-1].date



# Copyright (c) 2011 gocept gmbh & co. kg
# See also LICENSE.txt

from StringIO import StringIO
import datetime
import mock
import nagios.responsetime.logfile
import unittest


class LogFileTest(unittest.TestCase):

    def test_accepts_filename_instead_of_file(self):
        with mock.patch('__builtin__.open') as open:
            logfile = nagios.responsetime.logfile.LogFile('/path/to/file')
            open.assert_called_with('/path/to/file')
            self.assertEqual(open(), logfile.log_file)

    def test_parses_file_into_records(self):
        logfile = nagios.responsetime.logfile.LogFile(StringIO("""\
17/May/2011:14:09:42 +0200 "GET /RN342/@@searchresults.html?childrenSearchTermFranke&performSearch= HTTP/1.1" 0.282
17/May/2011:14:09:42 +0200 "GET /RN342/@@/kita-interfaces-IFibuUebergabe-kita_icon.png HTTP/1.1" 0.009
17/May/2011:14:09:43 +0200 "GET /RN342/einrichtung_4/54/obtainedServices/os-3 HTTP/1.1" 0.128
17/May/2011:14:09:44 +0200 "GET /RN342/einrichtung_4/54/obtainedServices/++resource++arrowup.gif HTTP/1.1" 0.012
"""))
        records = logfile.parse()
        self.assertEqual(4, len(records))
        r = records[0]
        self.assertEqual(datetime.datetime(2011, 5, 17, 14, 9, 42), r.date)
        self.assertEqual(
            '/RN342/@@searchresults.html?childrenSearchTermFranke'
            '&performSearch=', r.url)
        self.assertEqual(0.282, r.time)

    def test_filters_records_according_to_date(self):
        ANY = StringIO()
        logfile = nagios.responsetime.logfile.LogFile(ANY)

        with mock.patch_object(logfile, 'parse') as parse:
            old = mock.Mock()
            old.date = datetime.datetime(2011, 5, 17, 14, 0)
            new = mock.Mock()
            new.date = datetime.datetime(2011, 5, 17, 15, 0)
            parse.return_value = [old, new]
            self.assertEqual([new], logfile.records_since(
                datetime.datetime(2011, 5, 17, 14, 0)))


class StatisticsTest(unittest.TestCase):

    def setUp(self):
        from nagios.responsetime.logfile import Record
        from datetime import datetime
        self.stats = nagios.responsetime.logfile.Statistics([
            Record(date=datetime(2011, 5, 17, 14, 1), time=0.5),
            Record(date=datetime(2011, 5, 17, 14, 2), time=1),
            Record(date=datetime(2011, 5, 17, 14, 3), time=1),
            Record(date=datetime(2011, 5, 17, 14, 4), time=1.5),
        ])

    def test_statistics(self):
        self.assertEqual(4, self.stats.count)
        self.assertEqual(0.5, self.stats.min)
        self.assertEqual(1.5, self.stats.max)
        self.assertEqual(1, self.stats.mean)
        self.assertAlmostEqual(0.4, self.stats.stddev, 1)
        self.assertEqual(datetime.datetime(2011, 5, 17, 14, 4),
                         self.stats.last_date)



# Copyright (c) 2011 gocept gmbh & co. kg
# See also LICENSE.txt

import datetime
import mock
import nagios.responsetime.check
import nagiosplugin.cookie
import nagiosplugin.state
import os
import pkg_resources
import unittest


class FakeStatistics(object):

    min = 5
    max = 7
    mean = 5.5
    stddev = 1.25
    count = 27
    last_date = datetime.datetime(2011, 5, 19, 10, 51, 37)


class CheckResponseTimeTest(unittest.TestCase):

    def setUp(self):
        ANY = mock.Mock()
        self.check = nagios.responsetime.check.CheckResponseTime(ANY, ANY)

    def tearDown(self):
        # XXX We should do something clever to prevent the cookie from ending
        # up in $HOME.
        cookie = nagiosplugin.cookie.Cookie('nagios.responsetime')
        os.unlink(cookie.filename)

    def test_status_is_always_ok(self):
        with mock.patch_object(self.check, 'get_statistics') as get:
            get.return_value = FakeStatistics()
            self.check.obtain_data()

        self.assertEqual(
            nagiosplugin.state.Ok,
            reduce(nagiosplugin.state.reduce, self.check.states()))

    def test_performance_data_is_computed(self):
        with mock.patch_object(self.check, 'get_statistics') as get:
            get.return_value = FakeStatistics()
            self.check.obtain_data()

        self.assertEqual(
            [27, 7, 5.5, 5, 1.25],
            [m.value
             for m in sorted(self.check.measures, key=lambda m: m.name)])

    def test_repeated_call_remembers_data_seen(self):
        self.check.log_file = mock.sentinel.log_file
        with mock.patch_object(self.check, 'get_statistics') as get:
            stats = FakeStatistics()
            get.return_value = stats

            self.check.obtain_data()
            get.assert_called_with(
                mock.sentinel.log_file, datetime.datetime.min)
            self.check.obtain_data()
            get.assert_called_with(mock.sentinel.log_file, stats.last_date)


class EndtoEndTest(unittest.TestCase):

    def run_check(self, argv):
        main = nagiosplugin.Controller(
            nagios.responsetime.check.CheckResponseTime, argv)
        main.run()
        return main.format() + main.stderr

    def test_logfile_option_is_mandatory(self):
        self.assertEqual('RESPONSE UNKNOWN - -f|--logfile is required\n',
                         self.run_check([]))

    def test_endtoend(self):
        self.assertEqual(
            'RESPONSE OK | min=0.007 max=0.289 mean=0.118111111111'
            ' stddev=0.111523613373\n| count=18\n',
            self.run_check(['-f', pkg_resources.resource_filename(
                'nagios.responsetime.tests', 'fixtures/example.log')]))



# Copyright (c) 2011 gocept gmbh & co. kg
# See also LICENSE.txt

import datetime
import nagios.responsetime.logfile
import nagios.responsetime.util
import nagiosplugin
import nagiosplugin.measure


class CheckResponseTime(nagiosplugin.Check):

    name = 'Response time check'
    version = '0.1'

    log_file = None

    def __init__(self, optparser, logger):
        super(CheckResponseTime, self).__init__(optparser, logger)
        self.logger = logger
        optparser.add_option('-f', '--logfile', help='nginx timing log file',
                             default=None)

    def process_args(self, opts, args):
        self.log_file = opts.logfile
        if not self.log_file:
            return '-f|--logfile is required'

    def obtain_data(self):
        with nagios.responsetime.util.datetime_cookie(
                'nagios.responsetime') as cookie:
            last_seen_date = cookie.get(default=datetime.datetime.min)
            stats = self.get_statistics(self.log_file, last_seen_date)
            self.measures = self.convert_to_measures(stats)
            cookie.set(stats.last_date)

    def get_statistics(self, log_file, last_seen_date):
        logfile = nagios.responsetime.logfile.LogFile(log_file)
        return nagios.responsetime.logfile.Statistics(
            logfile.records_since(last_seen_date))

    def convert_to_measures(self, stats):
        return [nagiosplugin.measure.Measure(name, getattr(stats, name))
                for name in ('min', 'max', 'mean', 'stddev', 'count')]


def main():
    nagiosplugin.Controller(CheckResponseTime)()



# Copyright (c) 2011 gocept gmbh & co. kg
# See also LICENSE.txt

import os.path
from setuptools import setup, find_packages


def read(*path):
    return open(os.path.join(*path)).read() + '\n\n'

setup(
    name='nagios.responsetime',
    version='1.0rc1',
    author='gocept',
    author_email='mail@gocept.com',
    description='A Nagios plugin that collects response times from logs.',
    long_description = (
        read('README.txt') +
        '.. contents::\n\n' +
        read('CHANGES.txt')
        ),
    url='http://pypi.python.org/pypi/nagios.responsetime',
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data=True,
    zip_safe=False,
    license='ZPL 2.1',
    namespace_packages=['nagios'],
    entry_points="""
        [console_scripts]
        check_responsetime = nagios.responsetime.check:main
    """,
    install_requires=[
        'setuptools',
        'mock',
        'nagiosplugin',
        ])



# Copyright (c) 2011 gocept gmbh & co. kg
# See also LICENSE.txt

from contextlib import contextmanager
import datetime
import nagiosplugin.cookie
import time


class DatetimeCookie(nagiosplugin.cookie.Cookie):

    def get(self, default=None):
        raw = super(DatetimeCookie, self).get(default)
        if raw is default:
            return default
        return datetime.datetime.fromtimestamp(float((raw or '0').strip()))

    def set(self, value=None):
        if value:
            value = str(time.mktime(value.timetuple()))
        super(DatetimeCookie, self).set(value)


@contextmanager
def datetime_cookie(filename, dir=None):
    c = DatetimeCookie(filename, dir)
    yield c
    c.close()
